# OB - Framework to program bots
#
# ob/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    providing load/save methods.
    
"""

__version__ = 14

## imports

import os
import time
import types

## defines

basic_types = [dict, list, str, int, float, bool, None]
core_types = [list, str, int, float, bool, None]
homedir = os.path.expanduser("~")
starttime = time.time()
workdir = os.path.join(homedir, ".ob")

## imports

import _thread
import datetime
import logging
import ob
import os
import json
import sys
import threading
import types

## exceptions

class EKEY(Exception):
 
    pass

class ENOCLASS(Exception):

   pass

cache = {}

## decorators

def locked(func, *args, **kwargs):

    lock = _thread.allocate_lock()

    def lockedfunc(*args, **kwargs):
        lock.acquire()
        res = None
        try:
            res = func(*args, **kwargs)
        finally:
            try:
                lock.release()
            except:
                pass
        return res
    return lockedfunc

## classes

class Object(object):

    def __repr__(self):
        return "%s.%s at %s" % (
            self.__class__.__module__,
            self.__class__.__name__,
            str(hex(id(self)))
        )

    def __str__(self):
        return json.dumps(self, default=self.smooth, indent=4, sort_keys=True)

    def cdir(self, path):
        res = "" ; padje = ""
        for p in path.split(os.sep):
            res += "%s%s" % (p, os.sep)
            padje = os.path.abspath(os.path.normpath(res))
            if not os.path.exists(padje):
                try:
                    logging.warn("mkdir %s" % padje)
                    os.mkdir(padje)
                except (IsADirectoryError, NotADirectoryError, FileExistsError):
                    pass
        return padje

    def load(self, path, *args):
        assert path
        import json
        if path in cache:
            return cache[path]
        p = os.path.abspath(os.path.join(ob.workdir, path)) 
        logging.debug("load %s" % path)
        with open(p) as f:
            data = json.load(f)
            self.update(data)
        cache[path] = self
        return self

    def pure(self):
        return json.dumps(self, default=self.smooth, sort_keys=True)

    def save(self, path="", stime="", *args):
        if not path:
            self._type = self.get_type(self)
            path = os.path.join(self._type, stime or self.rtime())
        logging.info("save %s" % path)
        p = os.path.join(ob.workdir, path)
        d, fn = os.path.split(p)
        if not os.path.isdir(d):
            self.cdir(d)
        with open(p, "w") as f:
            json.dump(self, f, default=self.smooth, indent=4, sort_keys=True)
        return path

    def get_type(self, obj=None):
        if not obj:
            obj = self
        return obj.__class__.__module__ + "." + obj.__class__.__qualname__

    def rtime(self):
        return str(datetime.datetime.now()).replace(" ", os.sep)

    def smooth(self, obj):
        if isinstance(obj, Object):
            return obj
        return repr(obj)

    def upgrade(self, d):
        for k, v in d.items():
            if v:
                self[k] = v

## classes

class Obj(Object, dict):

    def __getattr__(self, k):
        if k in self:
            return self[k]
        return super().__getattribute__(k)

    def __setattr__(self, k, v):
        self[k] = v

    def last(self, path=""):
        from ob.db import Db
        db = Db()
        fn, disk = db.last(path or self.get_type())
        if disk:
            self.update(disk)
        return self

    def match(self, m):
        for k, v in self.items():
            if str(v).lower().find(m.lower()) != -1:
                return k

    def set(self, k, v):
        self[k] =  v
