# OB - Framework to program bots
#
# ob/bot.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" ob.bots package """

__version__ = 1

## imports

from ob import Obj
from ob.fleet import Fleet
from ob.handler import Handler
from ob.loader import Loader
from ob.shell import defaults, set_completer
from ob.users import Users

import logging
import os
import sys
import time
import threading

## instances

fleet = Fleet()
users = Users()

## exceptions

class ENOTIMPLEMENTED(Exception):

    pass

## classes

class Cfg(Obj):

    pass

class Bot(Handler):

    def __init__(self):
        super().__init__()
        self._ready = threading.Event()
        self.channels = []

    def announce(self, txt):
        pass

    def cmd(self, txt):
        users.oper("root@shell")
        event = super().event(txt, "root@shell")
        self.dispatch(event)
        return event

    def raw(self, txt):
        print(txt)

    def ready(self):
        self._ready.set()

    def register(self, k, v):
        self.handlers[k] = v

    def say(self, channel, txt):
        self.raw(txt)

    def start(self):
        import ob.handler
        ob.handler.names.last()
        set_completer(sorted(list(self.handlers.keys()) + os.listdir(ob.workdir)))
        fleet.add(self)
        super().start()
        return self

    def wait(self):
        self._ready.wait()
