# OB - Framework to program bots
#
# ob/cli.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" command line interface. """

__version__ = 1

## imports

from ob.bot import Bot
from ob.shell import set_completer
from ob.utils import cdir

import ob
import os
import sys
import threading

## classes

class CLI(Bot):

    def __init__(self):
        super().__init__()
        self._prompted = False
        self._ready = threading.Event()
        self._sock = sys.stdout

    def announce(self, txt):
        self.raw(txt)

    def dispatch(self, event):
        super().dispatch(event)
        if self._prompted:
            self.prompt()

    def event(self, txt):
        return super().event(txt, "root@shell")

    def get_event(self):
        try:
            return self.event(input())
        except EOFError:
            return None            

    def prompt(self):
        self._sock.write("> ")
        self._sock.flush()

    def raw(self, txt):
        self._sock.write(str(txt))
        self._sock.write("\n")
        self._sock.flush()

    def ready(self):
        self._ready.set()

    def say(self, channel, txt):
        self.raw(txt)

    def start(self):
        super().start()
        if self._prompted:
            self.prompt()

    def wait(self):
        self._ready.wait()
