# OB - Framework to program bots
#
# ob/cmds/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    basic commands.
    
"""

## imports

from ob import Obj, starttime
from ob.cmds.loader import load, unload
from ob.db import Db
from ob.fleet import Fleet
from ob.thr import Launcher
from ob.utils import elapsed

import ob
import os
import json
import time
import _thread

## defines

psformat = "%-8s %-60s"

## instances

bots = Fleet()
launcher = Launcher()

## commands

def cfg(event):
    """ cfg <type>. """
    if not event.args:
        event.reply("cfg <prefix>")
        return
    name = "ob.%s.Cfg" % event.args[0]
    try:
        cfg = Obj().last(name)
        event.reply(cfg)
        return
    except FileNotFoundError:
        event.reply("%s config not found" % event.args[0])
    else:
        try:
            event.reply(bots.bots[0].main)
        except IndexError:
            pass

def cmds(event):
    bot = bots.get_bot(event.orig)
    event.reply(",".join(sorted(bot.handlers.keys())))

def ed(event):
    db = Db()
    try:
        fn, obj = db.last(event.args[0])
    except IndexError:
        event.reply("ed <type>")
        return
    except IsADirectoryError as ex:
        event.reply("%s %s" % (event.txt, ex.strerror.lower()))
        return
    if not obj:
        event.reply("no match found for %s" % " ".join(event.args))
        return
    if len(event.args) == 1:
        event.reply(obj)
        return 
    key = event.args[1]
    if key not in obj:
        event.reply("no %s attribute available." % key)
        return
    if len(event.args) == 2:
        event.reply('%s=%s' % (key, obj[key]))
        return
    if len(event.args) == 3:
        val = event.args[2]
    elif len(event.args) >= 3:
        val = event.args[2:]
    try:
        val = int(val)
    except:
        try:
            val = float(val)
        except:
            pass
    t = type(obj[key])
    tt = type(val)
    if t != tt:
        event.reply("%s != %s" % (tt, t))
        return
    if t == str:
        obj[key] = val
    elif t == list:
        obj[key] = val
    elif val in ["True", "true"]:
        obj[key] = True
    elif val in ["False", "false"]:
        obj[key] = False
    else:
        try:
            val = '"%s"' % val
            obj[key] = json.loads(val)
        except json.decoder.JSONDecodeError:
            event.reply("%s is not JSON" % val)
        except (SyntaxError, ValueError, json.decode.JSONDecodeError) as ex:
            event.reply("%s: %s" % (str(ex), val))
        return
    obj.save(fn)
    event.reply('ok %s="%s"' % (key, obj[key]))

def fleet(event):
    try:
        nr = int(event.args[0])
    except (IndexError, ValueError) as ex:
        event.reply([type(b) for b in bots.bots])
        return
    try:
        event.reply(str(bots.bots[nr]))
    except IndexError:
        event.reply("less than %s bots available." % nr)

def ls(event):
    if not event.args:
        event.reply("\n".join(sorted(os.listdir(ob.workdir))))
    else:
        m = event.args[0].lower()
        event.reply("\n".join([x for x in os.listdir(ob.workdir) if m in x.lower()]))

def ps(event):
    result = []
    for thr in sorted(launcher.running(), key=lambda x: x.getName()):
        obj = Obj()
        obj.update(vars(thr))
        if obj.get("sleep", None):
            up = obj.sleep - int(time.time() - obj._time.latest)
        else:
            up = int(time.time() - starttime)
        thrname = thr.getName()
        result.append((up, thrname, obj))
    nr = 0
    for up, thrname, obj in sorted(result, key=lambda x: x[0]):
        res = "%s  %s" % (nr, psformat % (elapsed(up), thrname[:30]))
        event.reply(res.rstrip())
        nr += 1

def test(event):
    event.reply("yoo %s" % event.origin)
