# OB - Framework to program bots
#
# ob/cmds/db.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    providing load/save methods.
    
"""

__version__ = 1

## imports

from ob import workdir
from ob.db import Db

import os

## instances

db = Db()

## commands

def deleted(event):
    """ deleted <prefix> [match] - show objects with the deleted flag set. """
    if not event.args:
        event.reply("deleted what?")
        return
    for fn, obj in db.find(*event.args):
        if "deleted" in obj and not obj.deleted:
            continue
        event.display(obj)

def find(event):
    """ find <prefix> [match] - find objects with prefix and matching fields. """
    if not event.args:
        event.reply("find what?")
        return
    nr = 0
    for fn, obj in db.find(*event.args, **event.want):
        ok = False
        if "d" in event.options:
            event.reply(obj)
        elif "f" in event.options:
            ok = event.display(obj, txt=str(nr), full=True)
        else:
            ok = event.display(obj, txt=str(nr))
        if ok:
            nr += 1

def last(event):
    """ last <prefix> [match] - show last saved object. """
    if not event.args:
        event.reply("last what?")
        return
    fn, obj = db.last(*event.args)
    event.reply(obj)

def rm(event):
    """ rm <prefix> [match] - set deleted flag. """
    if not event.args:
        event.reply("remove what?")
        return
    nr = 0
    for fn, obj in db.find(*event.args):
        nr = 0
        obj.deleted = True
        obj.save()
        nr += 1
    event.reply("ok %s" % nr)
