# OB - Framework to program bots
#
# ob/db.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    cursor on the datastore, works with timestamped filenames.
    
"""

__version__ = 1

## imports

from ob import Obj
from ob.register import Register

import functools
import logging
import ob
import os

## classes

class Db(Obj):

    def __init__(self):
        super().__init__()
        self.fns = Register()
        if not os.path.exists(ob.workdir):
            self.cdir(ob.workdir)
        for fn in os.listdir(ob.workdir):
            name = fn.split(".")[-1].lower()
            self.fns.register(name, fn)

    def filenames(self, path):
        res = []
        p = os.path.join(ob.workdir, path.strip())
        for rootdir, dirs, files in os.walk(p, topdown=True):
            if p not in rootdir:
                continue
            for fn in files:
                if fn[-1] in ["~"]:
                    continue
                yield os.path.join(rootdir, fn).split(ob.workdir)[-1][1:]

    def deleted(self, path):
        for fn in sorted(self.filenames(path)):
            obj = Obj().load(fn)
            if "deleted" in dir(obj) and obj.deleted:
                yield fn, obj
        
    def find(self, path, match="", *args, **kwargs):
        if not path:
            return []
        want = path.split(".")[-1]
        fns = self.fns.get(want, [])
        if not fns:
            fns = [path, ]
        for name in fns:
            for fn in sorted(self.filenames(name)):
                obj = Obj().load(fn)
                if "deleted" in dir(obj) and obj.deleted:
                    continue
                if kwargs:
                    go = True
                    for k, v in kwargs.items():
                        vv = obj.get(k, None)
                        if vv and v not in vv:
                            go = False
                    if not go:
                        continue
                if obj.match(match):
                    yield fn, obj

    def last(self, path, match="", *args):
        fns = sorted(self.filenames(path))
        for fn in fns[::-1]:
            obj = Obj().load(fn)
            if obj.match(match):
                return fn, obj
        return None, None
