# OB - Framework to program bots
#
# ob/event.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" event class. """

__version__ = 1

## imports

from ob import Obj
from ob.fleet import Fleet
from ob.utils import parse
import threading

## defines

args = Obj()
args.find = Obj()
args.find.feed = ["link", "summary", "Date"]

## instances

fleet = Fleet()

## classes

class Event(Obj):

    def __init__(self, *args, txt=""):
        super().__init__(*args)
        self._ready = threading.Event()
        self.result = []
        self.txt = txt

    def __getattr__(self, k):
        try:
            return self.__getitem__(k)
        except KeyError:
            return ""
                     
    def display(self, obj, txt="", full=False):
        keys = None
        res = ""        
        keys = self.dkeys or obj.keys()
        for k in keys:
            if k.startswith("_"):
                continue
            v = obj.get(k, None)
            if v:
                if full:
                    res += "%s='%s' " % (k, v)
                else:
                    res += "%s " % v
        if res and txt:
            res = txt + " " +  res
            self.reply(res.strip())
            return True
        return False 

    def get_cmd(self):
        if self.txt.strip():
            return self.txt.strip().split()[0]
        return ""

    def ok(self):
        self.reply("ok %s" % self.get_cmd())

    def parse(self):
        txt = bytes(self.txt, "utf-8").decode("unicode_escape")
        quoted = False
        words = txt.split()
        self.args = []
        self.cmd = self.cmd or ""
        self.dkeys = []
        self.skip = []
        self.options = []
        self.rest = ""
        self.want = Obj()
        self.words = []
        self.counter = 0
        for word in words:
            self.counter += 1
            if self.counter == 1:
                if not not self.cmd:
                    self.cmd = word
                continue
            try:
                key, value = word.split("=", 1)
            except (IndexError, ValueError):
                key = ""
                value = word
            if value.startswith('"'):
                value = value[1:]
                key2 = key
                quoted = True
            if quoted and '"' in value:
                value = value.replace('"', '')
                quoted = False
                key = key2
            if quoted:
                self.words.append(value)
                continue
            else:
                self.words.append(value)
                value = " ".join(self.words)
            self.words = []
            if key and value:
                self.want[key] = value
                self.args.append(key)
                self.dkeys.append(key)
            elif value:
                if "," in value:
                    self.dkeys = value.split(",")
                    continue
                elif value.endswith("-"):
                    self.skip.append(value[:-1])
                elif value.startswith("-"):
                    self.options.append(value[1:])
                self.args.append(value)
                if self.counter > 2:
                    self.dkeys.append(value)
        if self.args:
            self.rest = " ".join(self.args)
        return self

    def ready(self):
        self._ready.set()

    def reply(self, txt, dofleet=True):
        self.result.append(txt)
        if dofleet:
            fleet.say(self.orig, self.channel, txt)

    def show(self, bot):
        for txt in self.result:
            bot.say(self.channel, txt)

    def wait(self):
        self._ready.wait()

