# OB - Framework to program bots
#
# ob/handler.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" queued event handler. """

## imports

from ob import Obj
from ob.event import Event
from ob.loader import Loader
from ob.register import Register
from ob.thr import Launcher, Worker
from ob.trace import get_exception
from ob.utils import stripped

import logging
import os
import queue
import time
import threading
import types

## classes

class Names(Obj):

    pass

class Handler(Launcher, Loader):

    def __init__(self):
        super().__init__()
        self.cbs = Register()
        self.cc = "!"
        self.handlers = Obj()
        self.names = names
        self.queue = queue.Queue()
        self.running = False
        self.stopped = False

    def callback(self, event):
        for cb in self.get_cbs(event.cb):
            self.launch(cb, event)

    def cancel(self):
        self.stopped = True
        self.queue.put(None)
        
    def dispatch(self, event):
        event.parse()
        func = self.get_cmd(event)
        if func:
            event.func = func
            logging.debug("dispatch %s %s" % (event.origin, func.__name__))
            func(event)
            if event.batch:
                event.show(self)
        event.ready()
        
    def event(self, txt="", origin=""):
        e = Event(txt=txt)
        e.cc = self.cc
        e.origin = origin
        e.orig = repr(self)
        return e

    def get_cbs(self, name):
        return self.cbs.get(name, [])

    def get_cmd(self, event):
        cmd = event.get_cmd()
        return self.get_handler(cmd)

    def get_event(self):
        return self.queue.get()

    def get_handler(self, cmd):
        val = self.handlers.get(cmd)
        if not val:
            modname = names.get(cmd)
            if modname:
                mod = self.direct(modname)
                self.load_mod(modname, mod, False)
            val = self.handlers.get(cmd)
        return val

    def put(self, event):
        self.queue.put(event)

    def load_mod(self, modname, mod, service):
        super().load_mod(modname, mod, service)
        for key, func, modname in self.scan(mod):
            if "callback" in dir(func) and func.callback:
                self.cbs.register(func.callback, func)
            else:
                self.handlers[key] = func
                names[key] = modname

    def loop(self):
        while not self.stopped:
            event = self.get_event()
            if not event:
                break
            self.launch(self.dispatch, event)
        logging.warn("stop %s" % repr(self))

    def scan(self, mod):
        for key in dir(mod):
            if key.startswith("_"):
                continue
            func = getattr(mod, key, None)
            if func and type(func) == types.FunctionType:
                if "event" in func.__code__.co_varnames:
                    yield (key, func, mod.__name__)

    def start(self):
        self.launch(self.loop)
           
    def stop(self): 
        self.stopped = True
        self.queue.put(None)

    def unload(self, modname):
        super().unload(modname)
        for key, func, name in self.scan(modname):
            if name == modname:
                del self[key]

names = Names()
names.update({
    "begin": "ob.cmds.clock",
    "cfg": "ob.cmds",
    "cmds": "ob.cmds",
    "deleted": "ob.cmds.db",
    "ed": "ob.cmds",
    "end": "ob.cmds.clock",
    "exit": "ob.cmds.system",
    "fetcher": "ob.cmds.rss",
    "find": "ob.cmds.db",
    "fleet": "ob.cmds",
    "friday": "ob.cmds.entry",
    "holiday": "ob.cmds.entry",
    "house": "ob.cmds.entry",
    "kill": "ob.cmds.system",
    "last": "ob.cmds.db",
    "load": "ob.cmds.loader",
    "log": "ob.cmds.entry",
    "ls": "ob.cmds",
    "mbox": "ob.cmds.email",
    "meet": "ob.cmds.users",
    "monday": "ob.cmds.entry",
    "perm": "ob.cmds.users",
    "ps": "ob.cmds",
    "quit": "ob.cmds.irc",
    "reboot": "ob.cmds.system",
    "rm": "ob.cmds.db",
    "rmperm": "ob.cmds.users",
    "rss": "ob.cmds.rss",
    "saturday": "ob.cmds.entry",
    "shop": "ob.cmds.entry",
    "stat": "ob.cmds.stats",
    "stats": "ob.cmds.stats",
    "sunday": "ob.cmds.entry",
    "test": "ob.cmds",
    "thursday": "ob.cmds.entry",
    "timer": "ob.cmds.clock",
    "todo": "ob.cmds.entry",
    "tuesday": "ob.cmds.entry",
    "unload": "ob.cmds.system",
    "uptime": "ob.cmds.system",
    "user": "ob.cmds.users",
    "version": "ob.cmds.system",
    "wednesday": "ob.cmds.entry",
    "wisdom": "ob.cmds.wisdom"
})
