# OB - Framework to program bots
#
# ob/init/rest.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 16-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" main program. """

__version__ = 1

## imports

from ob import Obj
from ob.thr import Launcher
from ob.trace import get_exception

import http
import http.server
import ob
import os
import logging
import socket
import time

## defines

defaults = Obj()
defaults.port = 10102

## instances

launcher = Launcher()

## classes

class Cfg(Obj):

    pass


## init

def start():
    cfg = Cfg(defaults).last()
    try:
        server = REST((socket.gethostname(), int(cfg.port)), RESTHandler)
        server.start()
        logging.error("listening on http://%s:%s" % (socket.gethostname(), cfg.port))
        return server
    except OSError as ex:
        logging.error(ex.strerror)

## classes

class REST(http.server.HTTPServer, Obj):

    allow_reuse_address = True
    daemon_thread = True

    def __init__(self, *args, **kwargs):
        http.server.HTTPServer.__init__(self, *args, **kwargs)
        Obj.__init__(self)
        self.host = args[0]
        self.last = time.time()
        self.starttime = time.time()
        self.stopped = False

    def exit(self):
        self.status = ""

    def server(self):
        self.serve_forever()

    def start(self):
        logging.warn("rest on http://%s:%s" % self.host)
        launcher.launch(self.server)

    def request(self):
        self.last = time.time()

    def error(self, request, addr):
        logging.info('! error rest %s %s' % (request, addr))

class RESTHandler(http.server.BaseHTTPRequestHandler):

    def setup(self):
        http.server.BaseHTTPRequestHandler.setup(self)
        self.hostname = socket.gethostname()
        self.ip = self.client_address[0]
        self.size = 0

    def write_header(self, typestr='text/plain'):
        self.send_response(200)
        self.send_header('Content-type', '%s; charset=%s ' % (typestr, "utf-8"))
        self.send_header('Server', ob.__version__)
        self.end_headers()

    def do_GET(self):
        if self.path == "/":
            self.write_header(typestr="text/html")
            self.wfile.write(bytes("Welcome to the OB-REST server<br><br>\n", "utf-8"))
            url = '<a href="%s">%s</a><br>\n'
            for fn in os.listdir(ob.workdir):
                txt = "http://%s:%s/%s" % (self.hostname, "10102", fn)
                self.wfile.write(bytes(url % (txt, txt), "utf-8"))
            self.wfile.flush()
            return
        path = ob.workdir + os.sep + self.path
        if os.path.isdir(path):
            self.write_header(typestr="text/html")
            self.wfile.write(bytes("Welcome to the OB-REST server<br><br>\n", "utf-8"))
            url = '<a href="%s">%s</a><br>\n'
            for fn in os.listdir(path):
                txt = os.path.join(self.path, fn)
                self.wfile.write(bytes(url % (txt, txt), "utf-8"))
            self.wfile.flush()
            return
        try:
            f = open(path, "r")
            txt = f.read()
            f.close()
        except (TypeError, FileNotFoundError, IsADirectoryError) as ex:
            self.send_response(404)
            return
        self.write_header(typestr="text/plain")
        txt = txt.replace("\\n", "\n")
        txt = txt.replace("\\t", "\t")
        self.wfile.write(bytes(txt, "utf-8"))
        self.wfile.flush()

    def log(self, code):
        logging.info('! %s %s %s' % (self.address_string(), code, self.path))

