# OB - Framework to program bots
#
# ob/init/udp.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 24-06-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" udp client program. """

__version__ = 1

## imports

from ob import Obj
from ob.shell import parse_cli
from ob.thr import Launcher

import logging
import os
import socket
import time
import threading

## defines

class Cfg(Obj):

    pass

defaults = Obj()
defaults.cfg = "udp"
defaults.host = "localhost"
defaults.port = 5500
defaults.password = "boh"
defaults.seed = "blablablablablaz" # needs to be 16 chars wide
defaults.server = defaults.host
defaults.owner = ""

## instances

launcher = Launcher()

## init

def start():
    server = UDP()
    server.start()
    return server

## classes

class UDP(Obj):

    def __init__(self):
        super().__init__(self)
        self._stopped = False
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        self._sock.setblocking(1)
        self._starttime = time.time()
        self.cfg = Cfg(defaults).last()

    def start(self, *args, **kwargs):
        if not os.path.exists(os.path.join(ob.workdir, "config", "udp")):
            self.cfg.save()
        launcher.launch(self.server)

    def server(self):
        self._sock.bind((self.cfg.host, self.cfg.port))
        logging.error("udp at %s:%s" % (self.cfg.host, self.cfg.port))
        while not self._stopped:
            (txt, addr) = self._sock.recvfrom(64000)
            if self._stopped:
                break
            data = str(txt.rstrip(), "utf-8")
            if not data:
                break
            self.output(data, addr)

    def exit(self):
        self._stopped = True
        self._sock.settimeout(0.01)
        self._sock.sendto(bytes("bla", "utf-8"), (self.cfg.host, self.cfg.port))

    def output(self, txt, addr=None):
        try:
            (passwd, text) = txt.split(" ", 1)
        except:
            return
        text = text.replace("\00", "")
        if passwd == self.cfg.password:
            for bot in fleet.bots:
                bot.announce(text)

