# OB - Framework to program bots
#
# ob/loader.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 17-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" a loader is an object that can load modules. """

__version__ = 1

## imports

from ob import Obj 
from ob.trace import get_exception
from ob.shell import set_completer

import importlib
import logging
import os
import pkgutil
import types
import sys

## classes

class Loader(Obj):

    def __init__(self):
        super().__init__()
        self.table = Obj()
        self.main = Obj()
    
    def direct(self, name, package=None):
        logging.info("direct %s" % name)
        return importlib.import_module(name, package)

    def list(self, name):
        res = []
        mod = self.direct(name)
        for name in pkgutil.walk_packages(mod.__path__, name+"."):
            res.append(name[1])
        return res

    def load_mod(self, modname, mod, service=False):
        self.table[modname] = mod
        if service:
            func = getattr(mod, "start", None)
            if func:
                logging.warn("service %s" % modname)
                self.launch(func)
        return mod

    def scan(self, mod):
        for key in dir(mod):
            if key.startswith("_"):
                continue
            obj = getattr(mod, key, None)
            if "__wrapped__" in dir(obj):
                func = obj.__wrapped__
            else:
                func = obj
            if func and type(func) == types.FunctionType:
                if "event" in func.__code__.co_varnames:
                    yield (key, func, mod.__name__)

    def unload(self, modname):
        del self.table[modname]

    def walk(self, name, service=False):
        if not name:
            return
        try:
            mod = self.direct(name)
            self.load_mod(name, mod, service)
        except ModuleNotFoundError as ex:
            logging.error(str(ex))
            return
        if "__path__" in dir(mod):
            for name in pkgutil.walk_packages(mod.__path__, name+"."):
                try:
                    mod = self.direct(name[1], mod)
                    self.load_mod(name[1], mod, service)
                except ModuleNotFoundError as ex:
                    logging.error(str(ex))

names = Obj()
names.update({
    "begin": "ob.cmds.clock",
    "cfg": "ob.cmds",
    "cmds": "ob.cmds",
    "deleted": "ob.cmds.db",
    "ed": "ob.cmds",
    "end": "ob.cmds.clock",
    "exit": "ob.cmds.system",
    "fetcher": "ob.cmds.rss",
    "find": "ob.cmds.db",
    "fleet": "ob.cmds",
    "friday": "ob.cmds.entry",
    "holiday": "ob.cmds.entry",
    "house": "ob.cmds.entry",
    "kill": "ob.cmds.system",
    "last": "ob.cmds.db",
    "load": "ob.cmds.loader",
    "log": "ob.cmds.entry",
    "ls": "ob.cmds",
    "mbox": "ob.cmds.email",
    "meet": "ob.cmds.users",
    "monday": "ob.cmds.entry",
    "perm": "ob.cmds.users",
    "ps": "ob.cmds",
    "quit": "ob.cmds.irc",
    "reboot": "ob.cmds.system",
    "rm": "ob.cmds.db",
    "rmperm": "ob.cmds.users",
    "rss": "ob.cmds.rss",
    "saturday": "ob.cmds.entry",
    "shop": "ob.cmds.entry",
    "stat": "ob.cmds.stats",
    "stats": "ob.cmds.stats",
    "sunday": "ob.cmds.entry",
    "test": "ob.cmds",
    "thursday": "ob.cmds.entry",
    "timer": "ob.cmds.clock",
    "todo": "ob.cmds.entry",
    "tuesday": "ob.cmds.entry",
    "unload": "ob.cmds.system",
    "uptime": "ob.cmds.system",
    "user": "ob.cmds.users",
    "version": "ob.cmds.system",
    "wednesday": "ob.cmds.entry",
    "wisdom": "ob.cmds.wisdom"
})
