# OB - Framework to program bots
#
# ob/utils/__init__.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""

    basic (most needed) helper fuunctions.
    
"""

__version__ = 1

## imports

from ob import core_types

import datetime
import logging
import os
import sys

## exceptions

class ENOCLASS(Exception):

    pass

### functions

def create(d):
    if type(d) in core_types:
        return d
    name = ""
    try:
        name = d["__id__"].split(os.sep)[0]
    except KeyError:
        return d
    try:
        modname, clsname = name.rsplit(".", 1)
    except ValueError:
        return d
    mod = sys.modules.get(modname)
    if not mod:
        from ob.loader import Loader
        l = Loader()
        mod = l.direct(modname)
    cls = getattr(mod, clsname, None)
    if cls:
        o = cls(d)
    else:
        return d
    for k, v in o.items():
        o[k] = create(v)
    return o

def cdir(path):
    res = ""
    padje = ""
    for p in path.split(os.sep):
        res += "%s%s" % (p, os.sep)
        padje = os.path.abspath(os.path.normpath(res))
        if not os.path.exists(padje):
            try:
                logging.warn("mkdir %s" % padje)
                os.mkdir(padje)
            except (IsADirectoryError, NotADirectoryError, FileExistsError):
                pass
    return padje

def dumped(o):
    t = type(o)
    if t == str:
        return o
    if t in [list, tuple]:
        l = []
        for obj in o:
            l.append(dumped(obj))
        return l
    if "items" not in dir(o):
        return o
    oo = t()
    for k, v in o.items():
        try:
            v["__id__"] = get_type2(v)
        except:
            pass
        if k == "name":
            oo["name"] = v
        else:
            oo[k] = dumped(v)
    if "__id__" not in oo:
        oo["__id__"] = get_type(oo)
    return oo

def elapsed(seconds, short=True):
    txt = ""
    sub = str(seconds).split(".")[-1]
    nsec = float(seconds)
    year = 365*24*60*60
    week = 7*24*60*60
    day = 24*60*60
    hour = 60*60
    minute = 60
    years = int(nsec/year)
    nsec -= years*year
    weeks = int(nsec/week)
    nsec -= weeks*week
    days = int(nsec/day)
    nsec -= days*day
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    sec = nsec - minutes*minute
    if years:
        txt += "%sy" % years
    if weeks:
        days += weeks * 7
    if days:
        txt += "%sd" % days
    if years and short and txt:
        return txt
    if hours:
        txt += "%sh" % hours
    if days and short and txt:
        return txt
    if minutes:
        txt += "%sm" % minutes
    if hours and short and txt:
        return txt
    if sec == 0:
        txt += "0s"
    elif sec < 1 or not short:
        txt += "%.3fs" % sec
    else:
        txt += "%ss" % int(sec)
    txt = txt.strip()
    return txt

def fn_time(daystr):
    daystr = daystr.replace("_", ":")
    datestr = " ".join(daystr.split(os.sep)[-2:])
    datestr = datestr.split(".")[0]
    try:
        t = time.mktime(time.strptime(datestr, "%Y-%m-%d %H:%M:%S"))
    except ValueError:
        t = 0
    return t

def name(func):
    try:
        n = "%s.%s" % (func.__self__.__class__.__name__, func.__name__)
    except:
        n = func.__name__
    return n

def parse(obj):
    txt = bytes(obj.txt, "utf-8").decode("unicode_escape")
    quoted = False
    words = txt.split()
    obj.args = []
    obj.cmd = ""
    obj.skip = []
    obj.options = []
    obj.want = Obj()
    obj.words = []
    obj.counter = 0
    for word in words:
        obj.counter += 1
        if obj.counter == 1:
           obj.cmd = word
           continue
        try:
            key, value = word.split("=", 1)
        except (IndexError, ValueError):
            key = ""
            value = word
        if value.startswith('"'):
            value = value[1:]
            key2 = key
            quoted = True
        if quoted and '"' in value:
            value = value.replace('"', '')
            quoted = False
            key = key2
        if quoted:
            obj.words.append(value)
            continue
        else:
            obj.words.append(value)
            value = " ".join(obj.words)
        if key and value:
            obj.want[key] = value
        elif value:
            if value.endswith("-"):
                obj.skip.append(value)
            if value.startswith("-"):
                obj.options.append(value)
            obj.args.append(value)
    return obj

def stripped(jid):
    try:
        return str(jid).split("/")[0]
    except:
        return str(jid)

