# OB - Framework to program bots
#
# ob/utils/clock.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" timer, repeater. timed daemon. """

__version__ = 1

## imports

import datetime
import os
import time

## defines

weekdays = ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")
months = ("Jan", "feb", "Mar", "Apr", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")

year_formats = [
    "%b %H:%M",
    "%b %H:%M:%S",
    "%a %H:%M %Y",
    "%a %H:%M",
    "%a %H:%M:%S",
    "%Y-%m-%d",
    "%d-%m-%Y",
    "%d-%m",
    "%m-%d",
    "%Y-%m-%d %H:%M:%S",
    "%d-%m-%Y %H:%M:%S",
    "%d-%m %H:%M:%S",
    "%m-%d %H:%M:%S",
    "%Y-%m-%d %H:%M",
    "%d-%m-%Y %H:%M",
    "%d-%m %H:%M",
    "%m-%d %H:%M",
    "%H:%M:%S",
    "%H:%M"
]

## functions

def get_time(daystr):
    for f in year_formats:
        try:
            t = time.mktime(time.strptime(daystr, f))
            return t
        except:
            pass

def hms():
    return str(datetime.datetime.today()).split()[1].split(".")[0]

def now():
    return str(datetime.datetime.now()).split(".")[0]

def rtime():
    res =  str(datetime.datetime.now()).replace(" ", os.sep)
    return res

def spath(path):
    return os.sep.join(path.split(os.sep)[-3:])

def today():
    t = rtime().split(".")[0]
    ttime = time.strptime(t, "%Y-%m-%d/%H:%M:%S")
    result = time.mktime(ttime)
    return result

def to_day(daystring):
    previous = ""
    line = ""
    daystr = str(daystring)
    for word in daystr.split():
        if "-" in word:
            line += word + " "
        elif ":" in word:
            line += word
    if "-" not in line:
        line = day() + " " + line
    try:
        return get_time(line.strip())
    except ValueError:
        pass

def scan_time(daystring):
    d = time.localtime()
    tstr = ""
    dstr = ""
    ystr = ""
    wday = ""
    mstr = ""
    for word in daystring.split():
        if word in weekdays:
            wday = word
        elif word in months:
            month = word
        elif "-" in word:
            dstr = word
        elif ":" in word:
            tstr = word
    if not tstr:
        tstr = str(d.tm_hour) + ":" + str(d.tm_min)
    if wday:
        difftonow = weekdays.index(wday) - d.tm_wday
        if difftonow < 0:
            difftonow = 7 + difftonow
        d = time.localtime(time.mktime(time.localtime()) + difftonow * 24* 60 * 60)
    if not dstr:
        dstr = str(d.tm_mday) + "-" + str(d.tm_mon) + "-" + str(d.tm_year)
    line = dstr + " " + tstr
    try:
        return get_time(line.strip())
    except ValueError:
        pass

def year():
    return str(datetime.datetime.now().year)
