# OB - Framework to program bots
# -*- coding: utf-8 -*-
#
# ob/xmpp.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 25-06-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" xmpp bot (requires sleekxmpp). """

version = 1

## imports

from ob import Obj, __version__
from ob.bot import Bot
from ob.event import Event
from ob.thr import Launcher
from ob.trace import get_exception
from ob.users import Users
from ob.utils import stripped

gotsleek = False
try:
    import sleekxmpp
    gotsleek = True
except ImportError:
    pass

import getpass
import logging
import os
import ssl
import threading
import time

## defines

class Cfg(Obj):

    pass

opts = [
    ("-u", "--user", "string", "", "user", "jid of the user."),
    ("-6", "--ipv6", 'store_true', False, "ipv6", "enable ipv6."),
    ("", "--noresolver", 'store_true', False, "noresolver", "disable the resolver."),
    ("", "--openfire", 'store_true', False, "openfire", "use openfire mode."),
] 

default = Cfg()
default.ipv6 = False
default.openfire = False
default.nick = "obbot"
default.noresolver = False
default.password = ""
default.port = 5222
default.room = ""
default.user = ""

## instances

launcher = Launcher()
users = Users()

## functions

def stripped(jid):
    try:
        return str(jid).split("/")[0]
    except:
        return str(jid)

## exceptions

class ENODATA(Exception):

    pass

## classes

class XMPP(Bot):

    cc = ""

    def __init__(self,cfg={}):
        super().__init__()
        self._connected = threading.Event()
        self.cfg = Cfg(default)
        self.channels = []
        self.jid = None
        self.users = []

    def _connect(self, user, pw):
        self._makeclient(user, pw)
        self.cfg.server = user.split("@")[-1]
        logging.warn("connect %s" % user)
        if self.cfg.noresolver:
            self.client.configure_dns(None)
        if self.cfg.openfire:
            self.client.connect(use_ssl=True)
        else:
            self.client.connect(reattempt=False, use_tls=True, use_ssl=False)
                        
    def _makeclient(self, jid, password):
        if not gotsleek:
            print("ob-xmpp needs sleekxmpp installed.")
            return
        self.client = sleekxmpp.clientxmpp.ClientXMPP(jid, password, plugin_config={}, plugin_whitelist=[], escape_quotes=False, sasl_mech=None)
        self.client.reconnect_max_attempts = 3
        self.client._error = Obj()
        self.client.use_ipv6 = self.cfg.ipv6
        if self.cfg.openfire:
            self.client.ssl_version = ssl.PROTOCOL_SSLv3
        self.client.register_plugin(u'xep_0045')
        self.client.add_event_handler('connected', self.connected)
        self.client.add_event_handler('disconnected', self.disconnected)
        self.client.add_event_handler('error', self.errored)
        self.client.add_event_handler('failed_auth', self.failedauth)
        self.client.add_event_handler("message", self.messaged)
        self.client.add_event_handler("iq", self.iqed)
        self.client.add_event_handler('presence', self.presenced)
        self.client.add_event_handler('presence_dnd', self.presenced)
        self.client.add_event_handler('presence_xa', self.presenced)
        self.client.add_event_handler('presence_available', self.presenced)
        self.client.add_event_handler('presence_chat', self.presenced)
        self.client.add_event_handler('presence_away', self.presenced)
        self.client.add_event_handler('presence_unavailable', self.presenced)
        self.client.add_event_handler('presence_subscribe', self.presenced)
        self.client.add_event_handler('presence_subscribed', self.presenced)
        self.client.add_event_handler('presence_unsubscribe', self.presenced)
        self.client.add_event_handler('presence_unsubscribed', self.presenced)
        self.client.add_event_handler("session_bind", self.session_bind)
        self.client.add_event_handler("session_start", self.session_start)
        self.client.add_event_handler("ssl_invalid_cert", self.invalid_cert)
        self.client.add_event_handler("ssl_cert", self.ssl_cert)
        self.client.exception = self.exception
        
    def announce(self, txt):
        for channel in self.channels:
            self.say(channel, txt)
        for origin in self.users:
            self.say(origin, txt, type="chat")

    def connect(self, user, password, port=None):
        if not password:
            print("no password provided.")
            return False
        self.cfg.save()
        try:
            self._connect(user, password)
        except sleekxmpp.exceptions.XMPPError as ex:
            logging.error(ex)
        self._sock = self.client.socket
        self._connected.set()
        return True
 
    def connected(self, data):
        logging.warn("connected to %s" % self.cfg.server)

    def disconnected(self, data):
        logging.warn("disconnected from %s" % self.cfg.server)

    def errored(self, data):
        pass

    def exception(self, data):
        logging.error("error %s" % data)

    def failedauth(self, data):
        logging.info("failed auth %s" % self.cfg.server)

    def failure(self, data):
        logging.info("failure %s" % data)

    def groupchat_presence(self, data):
        e = Event()
        e.update(data)
        #if self.cfg.user in str(e["to"]):
        #    self.jid = e.jid

    def groupchat_subject(self, data):
        e = Event()
        e.update(data)
        #if self.cfg.nick in str(e["from"]):
        #    self.jid = e["to"]
        
    def invalid_cert(self, data):
        logging.info("invalid cert %s" % self.cfg.server)

    def ssl_cert(self, data):
        pass

    def iqed(self, data):
        logging.warn("iq %s" % data)

    def join(self, nick="", room=""):
        self.cfg.room = room or self.cfg.room
        self.cfg.nick = nick or self.cfg.nick
        assert self.cfg.room, self.cfg.nick
        self.cfg.save()
        logging.warn("join %s as %s" % (self.cfg.room, self.cfg.nick))
        if self.cfg.room not in self.channels:
            self.channels.append(self.cfg.room)
        self.client.plugin['xep_0045'].joinMUC(self.cfg.room,
                                    self.cfg.nick,
                                    wait=True)

    def messaged(self, data):
        if '<delay xmlns="urn:xmpp:delay"' in str(data):
            logging.warn("ignore %s" % data["From"])
            return
        txt = data["body"]
        m = Event(txt=txt)
        m.orig = repr(self)
        m.type = data["type"]
        if m["type"] == "error":
            logging.error("error %s" % m.error)
            return
        m.origin = str(data["from"])
        m.nick = m.origin.split("/")[-1]
        m.server = self.cfg.server
        m.channel = m.origin
        m.to = m.origin
        m.cb = "Message"
        if data["type"] == "groupchat":
            users.userhosts[m.nick] = stripped(str(m["from"]))
            m.channel = stripped(m.origin)
        if self.cfg.user == stripped(m.origin):
            logging.warn("ignore %s %s" % (m.type, m["from"]))
            return
        m.prompt = False
        if m.txt:
            if m.txt[0] != "!":
                m.txt = "log " + m.txt
            else:
                m.txt = m.txt[1:]
        m.parse()
        super().dispatch(m)

    def pinged(self, event):
        logging.info(event)

    def presenced(self, data):
        o = Event()
        o.type = data["type"]
        o.orig = repr(self)
        o.cc = ""
        o.origin = str(data["from"])
        o.jid = o.origin
        o.nick = o.origin.split("/")[-1]
        o.server = self.cfg.server
        o.channel = stripped(o.origin)
        if "txt" not in o:
            o.txt = ""
        o.element = "presence"
        if self.cfg.user in str(data["from"]):
            self._connected.set()
            logging.warn("ignore %s" % o.origin)
            return
        if o.type == 'subscribe':
            pres = Event({'to': o.origin, 'type': 'subscribed'})
            self.client.send_presence(dict(pres))
            pres = Event({'to': o.origin, 'type': 'subscribe'})
            self.client.send_presence(dict(pres))
        elif o.type == "unsubscribe":
            if o.origin in self.channels:
                self.channels.remove(o.origin)
            if o.origin in self.users:
                self.users.remove(o.origin)
            return
        elif o.type == "available":
            if o.origin not in self.users:
                self.users.append(o.origin)
        elif o.type == "unavailable":
            if o.origin in self.users:
                self.users.remove(o.origin) 
        o.cb = "Presence"
        users.userhosts[o.nick] = stripped(o.origin)
        logging.warn("presence %s" % o.origin)

    def say(self, channel, txt, type="chat", id=""):
        try:
            sleekxmpp.jid.JID(channel)
        except sleekxmpp.jid.InvalidJID:
            logging.warn("wrong jid %s" % channel)
            return
        if self.cfg.user in channel:
            logging.warn("not writing to self.")
            return
        if channel in self.channels:
            type = "groupchat"
        if type == "groupchat":
            channel = stripped(channel)
        self.client.send_message(mto=channel,
                                 mbody=str(txt),
                                 mtype=type)
                                 
    def session_bind(self, data):
        self.jid = str(data)
        logging.warn("jid is %s" % self.jid)

    def session_start(self, data):
        import sleekxmpp
        logging.info("session start %s" % data)
        try:
            self.client.send_presence()
            self.client.get_roster()
        except sleekxmpp.exceptions.XMPPError as ex:
            logging.error(ex)
        except sleekxmpp.exceptions.IqTimeout:
            logging.error("iq timeout %s" % self.cfg.server)
            k.unregister(self)
            self.reconnect()

    def sleek(self):
        self._connected.wait()
        try:
            self.client.process(block=True)
        except sleekxmpp.exceptions.XMPPError as ex:
            logging.debug(ex)
        except Exception as ex:
            logging.error(get_exception())

    def stop(self):
        if "client" in self and self.client:
            self.client.disconnect()
        super().stop()

    def start(self):
        super().start()
        self.cfg.last()
        self._nrreconnect = 2
        if not self.cfg.password:
            self.cfg.password = getpass.getpass(prompt="password: ")
            self.cfg.save()
        if not self.cfg.user:
            print("use -u to provide a user JID")
            return False
        if not self.connect(self.cfg.user, self.cfg.password):
            self._connected.set()
            return False
        launcher.launch(self.sleek)
        if self.cfg.room:
            self.join()
        return True
